from typing import Optional, Union, List, Dict, Literal

from pydantic import BaseModel, Field

from .ticket_models import TicketDetailInput


class AuthData(BaseModel):
    """
    Authentication credentials for OTOBO Webservice API.

    Attributes:
        SessionID (Optional[int]): Optional session identifier for existing sessions.
        UserLogin (str): Agent login name for authentication.
        Password (str): Agent password for authentication.
    """
    SessionID: Optional[int] = None
    UserLogin: str = Field(..., description="Agent login for authentication")
    Password: str = Field(..., description="Agent password for authentication")


class DynamicFieldParams(BaseModel):
    """
    Parameters for filtering dynamic ticket fields in search operations.

    Attributes:
        Empty (Optional[bool]): Match if field is empty.
        Equals (Optional[Union[int, str, List[Union[int, str]]]]): Exact match value(s).
        Like (Optional[str]): Pattern match string.
        GreaterThan (Optional[str]): Match values greater than this.
        GreaterThanEquals (Optional[str]): Match values greater than or equal to this.
        SmallerThan (Optional[str]): Match values less than this.
        SmallerThanEquals (Optional[str]): Match values less than or equal to this.
    """
    Empty: Optional[bool] = None
    Equals: Optional[Union[int, str, List[Union[int, str]]]] = None
    Like: Optional[str] = None
    GreaterThan: Optional[str] = None
    GreaterThanEquals: Optional[str] = None
    SmallerThan: Optional[str] = None
    SmallerThanEquals: Optional[str] = None


class TicketSearchParams(BaseModel):
    """
    Search filters for querying tickets via OTOBO Webservice.

    DynamicFields are remapped to 'DynamicField_<name>' in model_dump override.
    Attributes:
            TicketNumber: Optional[Union[str, List[str]]] = None
            Title: Optional[Union[str, List[str]]] = None
            Queues: Optional[List[str]] = None
            QueueIDs: Optional[List[int]] = None
            UseSubQueues: Optional[bool] = None
            Types: Optional[List[str]] = None
            TypeIDs: Optional[List[int]] = None
            States: Optional[List[str]] = None
            StateIDs: Optional[List[int]] = None
            StateType: Optional[Union[str, List[str]]] = None
            StateTypeIDs: Optional[List[int]] = None
            Priorities: Optional[List[str]] = None
            PriorityIDs: Optional[List[int]] = None
            Services: Optional[List[str]] = None
            ServiceIDs: Optional[List[int]] = None
            SLAs: Optional[List[str]] = None
            SLAIDs: Optional[List[int]] = None
            Locks: Optional[List[str]] = None
            LockIDs: Optional[List[int]] = None
            OwnerIDs: Optional[List[int]] = None
            ResponsibleIDs: Optional[List[int]] = None
            WatchUserIDs: Optional[List[int]] = None
            CustomerID: Optional[Union[str, List[str]]] = None
            CustomerIDRaw: Optional[Union[str, List[str]]] = None
            CustomerUserLogin: Optional[Union[str, List[str]]] = None
            CreatedUserIDs: Optional[List[int]] = None
            CreatedTypes: Optional[List[str]] = None
            CreatedTypeIDs: Optional[List[int]] = None
            CreatedPriorities: Optional[List[str]] = None
            CreatedPriorityIDs: Optional[List[int]] = None
            CreatedStates: Optional[List[str]] = None
            CreatedStateIDs: Optional[List[int]] = None
            CreatedQueues: Optional[List[str]] = None
            CreatedQueueIDs: Optional[List[int]] = None
            DynamicFields: Optional[Dict[str, DynamicFieldParams]] = None
            MIMEBase_From: Optional[str] = None
            MIMEBase_To: Optional[str] = None
            MIMEBase_Cc: Optional[str] = None
            MIMEBase_Subject: Optional[str] = None
            MIMEBase_Body: Optional[str] = None
            AttachmentName: Optional[str] = None
            FullTextIndex: Optional[bool] = None
            ContentSearch: Optional[str] = None
            ConditionInline: Optional[bool] = None
            ArticleCreateTimeOlderMinutes: Optional[int] = None
            ArticleCreateTimeNewerMinutes: Optional[int] = None
            ArticleCreateTimeNewerDate: Optional[str] = None
            ArticleCreateTimeOlderDate: Optional[str] = None
            TicketCreateTimeOlderMinutes: Optional[int] = None
            TicketCreateTimeNewerMinutes: Optional[int] = None
            TicketCreateTimeNewerDate: Optional[str] = None
            TicketCreateTimeOlderDate: Optional[str] = None
            TicketChangeTimeOlderMinutes: Optional[int] = None
            TicketChangeTimeNewerMinutes: Optional[int] = None
            TicketLastChangeTimeOlderMinutes: Optional[int] = None
            TicketLastChangeTimeNewerMinutes: Optional[int] = None
            TicketLastChangeTimeNewerDate: Optional[str] = None
            TicketLastChangeTimeOlderDate: Optional[str] = None
            TicketChangeTimeNewerDate: Optional[str] = None
            TicketChangeTimeOlderDate: Optional[str] = None
            TicketCloseTimeOlderMinutes: Optional[int] = None
            TicketCloseTimeNewerMinutes: Optional[int] = None
            TicketCloseTimeNewerDate: Optional[str] = None
            TicketCloseTimeOlderDate: Optional[str] = None
            TicketPendingTimeOlderMinutes: Optional[int] = None
            TicketPendingTimeNewerMinutes: Optional[int] = None
            TicketPendingTimeNewerDate: Optional[str] = None
            TicketPendingTimeOlderDate: Optional[str] = None
            TicketEscalationTimeOlderMinutes: Optional[int] = None
            TicketEscalationTimeNewerMinutes: Optional[int] = None
            TicketEscalationTimeNewerDate: Optional[str] = None
            TicketEscalationTimeOlderDate: Optional[str] = None
            SearchInArchive: Optional[str] = None
            OrderBy: Optional[Union[str, List[str]]] = None
            SortBy: Optional[Union[str, List[str]]] = None
    """
    TicketNumber: Optional[Union[str, List[str]]] = None
    Title: Optional[Union[str, List[str]]] = None
    Queues: Optional[List[str]] = None
    QueueIDs: Optional[List[int]] = None
    UseSubQueues: Optional[bool] = None
    Types: Optional[List[str]] = None
    TypeIDs: Optional[List[int]] = None
    States: Optional[List[str]] = None
    StateIDs: Optional[List[int]] = None
    StateType: Optional[Union[str, List[str]]] = None
    StateTypeIDs: Optional[List[int]] = None
    Priorities: Optional[List[str]] = None
    PriorityIDs: Optional[List[int]] = None
    Services: Optional[List[str]] = None
    ServiceIDs: Optional[List[int]] = None
    SLAs: Optional[List[str]] = None
    SLAIDs: Optional[List[int]] = None
    Locks: Optional[List[str]] = None
    LockIDs: Optional[List[int]] = None
    OwnerIDs: Optional[List[int]] = None
    ResponsibleIDs: Optional[List[int]] = None
    WatchUserIDs: Optional[List[int]] = None
    CustomerID: Optional[Union[str, List[str]]] = None
    CustomerIDRaw: Optional[Union[str, List[str]]] = None
    CustomerUserLogin: Optional[Union[str, List[str]]] = None
    CreatedUserIDs: Optional[List[int]] = None
    CreatedTypes: Optional[List[str]] = None
    CreatedTypeIDs: Optional[List[int]] = None
    CreatedPriorities: Optional[List[str]] = None
    CreatedPriorityIDs: Optional[List[int]] = None
    CreatedStates: Optional[List[str]] = None
    CreatedStateIDs: Optional[List[int]] = None
    CreatedQueues: Optional[List[str]] = None
    CreatedQueueIDs: Optional[List[int]] = None
    DynamicFields: Optional[Dict[str, DynamicFieldParams]] = None
    MIMEBase_From: Optional[str] = None
    MIMEBase_To: Optional[str] = None
    MIMEBase_Cc: Optional[str] = None
    MIMEBase_Subject: Optional[str] = None
    MIMEBase_Body: Optional[str] = None
    AttachmentName: Optional[str] = None
    FullTextIndex: Optional[bool] = None
    ContentSearch: Optional[str] = None
    ConditionInline: Optional[bool] = None
    ArticleCreateTimeOlderMinutes: Optional[int] = None
    ArticleCreateTimeNewerMinutes: Optional[int] = None
    ArticleCreateTimeNewerDate: Optional[str] = None
    ArticleCreateTimeOlderDate: Optional[str] = None
    TicketCreateTimeOlderMinutes: Optional[int] = None
    TicketCreateTimeNewerMinutes: Optional[int] = None
    TicketCreateTimeNewerDate: Optional[str] = None
    TicketCreateTimeOlderDate: Optional[str] = None
    TicketChangeTimeOlderMinutes: Optional[int] = None
    TicketChangeTimeNewerMinutes: Optional[int] = None
    TicketLastChangeTimeOlderMinutes: Optional[int] = None
    TicketLastChangeTimeNewerMinutes: Optional[int] = None
    TicketLastChangeTimeNewerDate: Optional[str] = None
    TicketLastChangeTimeOlderDate: Optional[str] = None
    TicketChangeTimeNewerDate: Optional[str] = None
    TicketChangeTimeOlderDate: Optional[str] = None
    TicketCloseTimeOlderMinutes: Optional[int] = None
    TicketCloseTimeNewerMinutes: Optional[int] = None
    TicketCloseTimeNewerDate: Optional[str] = None
    TicketCloseTimeOlderDate: Optional[str] = None
    TicketPendingTimeOlderMinutes: Optional[int] = None
    TicketPendingTimeNewerMinutes: Optional[int] = None
    TicketPendingTimeNewerDate: Optional[str] = None
    TicketPendingTimeOlderDate: Optional[str] = None
    TicketEscalationTimeOlderMinutes: Optional[int] = None
    TicketEscalationTimeNewerMinutes: Optional[int] = None
    TicketEscalationTimeNewerDate: Optional[str] = None
    TicketEscalationTimeOlderDate: Optional[str] = None
    SearchInArchive: Optional[str] = None
    OrderBy: Optional[Union[str, List[str]]] = None
    SortBy: Optional[Union[str, List[str]]] = None

    def model_dump(self, **kwargs):
        """
        Override to remap DynamicFields to individual keys.

        Converts DynamicFields dict into separate DynamicField_<name> entries in output.
        """
        data = super().model_dump(**kwargs)
        df = data.pop("DynamicFields", {}) or {}
        for name, params in df.items():
            data[f"DynamicField_{name}"] = params
        return data


class TicketCreateParams(TicketDetailInput):
    """
    Parameters for creating a new ticket, extends TicketDetailInput.
    """
    pass


class TicketGetParams(BaseModel):
    """
    Parameters for retrieving a ticket by ID with optional article and attachment controls.

    Attributes:
        TicketID (Optional[int]): ID of the ticket to fetch.
        DynamicFields (int): Include dynamic fields (1 to include).
        Extended (int): Include extended data (1 to include).
        AllArticles (int): Include all articles (1 to include).
        ArticleSenderType (Optional[List[str]]): Filter articles by sender type.
        ArticleOrder (Literal['ASC','DESC']): Order of articles, 'ASC' or 'DESC'.
        ArticleLimit (int): Max number of articles to return.
        Attachments (int): Include attachments metadata (1 to include).
        GetAttachmentContents (int): Include attachment contents (1 to include).
        HTMLBodyAsAttachment (int): Include HTML body as attachment (1 to include).
    """
    TicketID: Optional[int] = None
    DynamicFields: int = 1
    Extended: int = 1
    AllArticles: int = 1
    ArticleSenderType: Optional[List[str]] = None
    ArticleOrder: Literal["ASC", "DESC"] = 'ASC'
    ArticleLimit: int = 20
    Attachments: int = 0
    GetAttachmentContents: int = 1
    HTMLBodyAsAttachment: int = 1


class TicketHistoryParams(BaseModel):
    """
    Parameters for fetching the history of a ticket.

    Attributes:
        TicketID (str): Identifier of the ticket whose history is requested.
    """
    TicketID: str


class TicketUpdateParams(TicketDetailInput):
    """
    Parameters for updating an existing ticket, extends TicketDetailInput.

    Attributes:
        TicketID (Optional[int]): ID of the ticket to update.
        TicketNumber (Optional[str]): Number of the ticket to update.
    """
    TicketID: Optional[int] = None
    TicketNumber: Optional[str] = None



from typing import Any, List, Union, Optional
from pydantic import BaseModel


class AttachmentModel(BaseModel):
    """
    Model representing an attachment to be sent with a ticket operation.

    Attributes:
        Filename (str): Name of the file including extension.
        ContentType (Optional[str]): MIME type of the file (e.g., 'application/pdf').
        Content (Optional[str]): Base64-encoded content of the file.
    """
    Filename: str
    ContentType: Optional[str] = None
    Content: Optional[str] = None


class AttachmentResponseDetail(AttachmentModel):
    """
    Detailed model for an attachment returned by OTOBO.

    Attributes:
        ContentAlternative (Optional[str]): Alternative content representation.
        ContentID (Optional[str]): Identifier for inline content references.
        Filesize (Optional[str]): Human-readable file size (e.g., '15 KB').
        FilesizeRaw (Optional[int]): File size in bytes.
    """
    ContentAlternative: Optional[str] = None
    ContentID: Optional[str] = None
    Filesize: Optional[str] = None
    FilesizeRaw: Optional[int] = None


class TicketBase(BaseModel):
    """
    Common fields for ticket creation and updates.

    Attributes:
        Title (Optional[str]): Subject or title of the ticket.
        QueueID (Optional[int]): Numeric ID of the queue.
        Queue (Optional[str]): Name of the queue.
        LockID (Optional[int]): Numeric ID of the lock status.
        Lock (Optional[str]): Name of the lock status.
        TypeID (Optional[int]): Numeric ID of the ticket type.
        Type (Optional[str]): Name of the ticket type.
        ServiceID (Optional[Union[int,str]]): Numeric or string ID of the service.
        Service (Optional[str]): Name of the service.
        SLAID (Optional[Union[int,str]]): Numeric or string ID of the SLA.
        SLA (Optional[str]): Name of the SLA.
        StateID (Optional[int]): Numeric ID of the ticket state.
        State (Optional[str]): Name of the ticket state.
        PriorityID (Optional[int]): Numeric ID of the priority.
        Priority (Optional[str]): Name of the priority.
        OwnerID (Optional[int]): Numeric ID of the ticket owner.
        Owner (Optional[str]): User login of the ticket owner.
        ResponsibleID (Optional[int]): Numeric ID of the responsible agent.
        Responsible (Optional[str]): User login of the responsible agent.
        CustomerUser (Optional[str]): Login name of the customer user.
    """
    Title: Optional[str] = None
    QueueID: Optional[int] = None
    Queue: Optional[str] = None
    LockID: Optional[int] = None
    Lock: Optional[str] = None
    TypeID: Optional[int] = None
    Type: Optional[str] = None
    ServiceID: Optional[Union[int, str]] = None
    Service: Optional[str] = None
    SLAID: Optional[Union[int, str]] = None
    SLA: Optional[str] = None
    StateID: Optional[int] = None
    State: Optional[str] = None
    PriorityID: Optional[int] = None
    Priority: Optional[str] = None
    OwnerID: Optional[int] = None
    Owner: Optional[str] = None
    ResponsibleID: Optional[int] = None
    Responsible: Optional[str] = None
    CustomerUser: Optional[str] = None


class TicketCommon(TicketBase):
    """
    Extended ticket fields common to detailed output.

    Attributes:
        TicketID (Optional[int]): Unique identifier of the ticket.
        TicketNumber (Optional[str]): Human-readable ticket number.
        StateType (Optional[str]): Type of the state (e.g., 'open', 'closed').
        CustomerID (Optional[str]): Identifier for the customer.
        CustomerUserID (Optional[str]): Numeric ID of the customer user.
        ArchiveFlag (Optional[str]): Archive status flag.
        Age (Optional[int]): Age of the ticket in seconds.
        EscalationResponseTime (Optional[int]): Configured response escalation time.
        EscalationUpdateTime (Optional[int]): Configured update escalation time.
        EscalationSolutionTime (Optional[int]): Configured solution escalation time.
        EscalationTime (Optional[int]): Total escalation time configured.
        CreateBy (Optional[int]): User ID who created the ticket.
        ChangeBy (Optional[int]): User ID who last modified the ticket.
        Created (Optional[str]): Timestamp when the ticket was created.
        Changed (Optional[str]): Timestamp when the ticket was last changed.
    """
    TicketID: Optional[int] = None
    TicketNumber: Optional[str] = None
    StateType: Optional[str] = None
    CustomerID: Optional[str] = None
    CustomerUserID: Optional[str] = None
    ArchiveFlag: Optional[str] = None
    Age: Optional[int] = None
    EscalationResponseTime: Optional[int] = None
    EscalationUpdateTime: Optional[int] = None
    EscalationSolutionTime: Optional[int] = None
    EscalationTime: Optional[int] = None
    CreateBy: Optional[int] = None
    ChangeBy: Optional[int] = None
    Created: Optional[str] = None
    Changed: Optional[str] = None


class DynamicFieldItem(BaseModel):
    """
    Represents a dynamic field key-value pair for tickets.

    Attributes:
        Name (str): Name of the dynamic field.
        Value (Optional[Any]): Value assigned to the field.
    """
    Name: str
    Value: Optional[Any] = None


class ArticleDetail(BaseModel):
    """
    Detailed model of an article within a ticket.

    Attributes:
        CommunicationChannel (Optional[str]): Channel of communication (e.g., 'email').
        CommunicationChannelID (Optional[int]): Numeric ID of the channel.
        IsVisibleForCustomer (Optional[int]): Visibility flag for the customer.
        SenderTypeID (Optional[Union[int,str]]): Numeric or string ID of sender type.
        AutoResponseType (Optional[str]): Type of auto-response triggered.
        From (Optional[str]): Sender address or login.
        Subject (Optional[str]): Subject line of the article.
        Body (Optional[str]): Content body of the article.
        ContentType (Optional[str]): Content type MIME header.
        MimeType (Optional[str]): MIME type string.
        Charset (Optional[str]): Character set encoding.
        CreateTime (Optional[str]): Timestamp when the article was created.
        ChangeTime (Optional[str]): Timestamp of last modification.
        IncomingTime (Optional[int]): Unix timestamp of incoming message.
        To (Optional[str]): Recipient address or login.
        SenderType (Optional[str]): Description of sender type.
        IsEdited (Optional[int]): Edit flag indicator.
        Cc (Optional[str]): Carbon copy recipients.
        Bcc (Optional[str]): Blind carbon copy recipients.
        ReplyTo (Optional[str]): Reply-to address.
        InReplyTo (Optional[str]): In-reply-to message ID.
        References (Optional[str]): References header links.
        MessageID (Optional[str]): Message-ID header.
        ContentCharset (Optional[str]): Content charset header.
        ChangeBy (Optional[int]): User ID who modified the article.
        CreateBy (Optional[int]): User ID who created the article.
        IsDeleted (Optional[int]): Deletion flag indicator.
        ArticleID (Optional[int]): Unique article identifier.
        ArticleNumber (Optional[int]): Sequential article number.
        DynamicField (Optional[List[DynamicFieldItem]]): List of dynamic fields.
        Attachment (Optional[AttachmentResponseDetail]): Attached file details.
    """
    CommunicationChannel: Optional[str] = None
    CommunicationChannelID: Optional[int] = None
    IsVisibleForCustomer: Optional[int] = None
    SenderTypeID: Optional[Union[int, str]] = None
    AutoResponseType: Optional[str] = None
    From: Optional[str] = None
    Subject: Optional[str] = None
    Body: Optional[str] = None
    ContentType: Optional[str] = None
    MimeType: Optional[str] = None
    Charset: Optional[str] = None
    CreateTime: Optional[str] = None
    ChangeTime: Optional[str] = None
    IncomingTime: Optional[int] = None
    To: Optional[str] = None
    SenderType: Optional[str] = None
    IsEdited: Optional[int] = None
    Cc: Optional[str] = None
    Bcc: Optional[str] = None
    ReplyTo: Optional[str] = None
    InReplyTo: Optional[str] = None
    References: Optional[str] = None
    MessageID: Optional[str] = None
    ContentCharset: Optional[str] = None
    ChangeBy: Optional[int] = None
    CreateBy: Optional[int] = None
    IsDeleted: Optional[int] = None
    ArticleID: Optional[int] = None
    ArticleNumber: Optional[int] = None
    DynamicField: Optional[List[DynamicFieldItem]] = None
    Attachment: Optional[AttachmentResponseDetail] = None


class TicketDetailOutput(TicketCommon):
    """
    Full ticket model returned by OTOBO including articles and dynamic fields.

    Attributes:
        Article (Union[ArticleDetail,List[ArticleDetail]]): Single or list of article details.
        DynamicField (List[DynamicFieldItem]): List of dynamic field items on the ticket.
    """
    Article: Union[ArticleDetail, List[ArticleDetail]]
    DynamicField: List[DynamicFieldItem]


class TicketDetailInput(BaseModel):
    """
    Model for creating or updating a ticket, includes optional details.

    Attributes:
        Ticket (Optional[TicketCommon]): Core ticket fields to set.
        Article (Optional[Union[ArticleDetail,List[ArticleDetail]]]): Article(s) to attach.
        DynamicField (Optional[List[DynamicFieldItem]]): Dynamic fields to set.
        Attachment (Optional[AttachmentModel]): Single attachment to include.
    """
    Ticket: Optional[TicketCommon] = None
    Article: Optional[Union[ArticleDetail, List[ArticleDetail]]] = None
    DynamicField: Optional[List[DynamicFieldItem]] = None
    Attachment: Optional[AttachmentModel] = None
